@@ -1,7 +1,7 @@
-/* config.c - MemTest-86  Version 3.0
+/* config.c - MemTest-86  Version 3.1
  *
  * Released under version 2 of the Gnu Public License.
- * By Chris Brady, cbrady@sgi.com
+ * By Chris Brady
  */
 #include "test.h"
 #include "screen_buffer.h"
@@ -10,7 +10,6 @@
 
 extern int bail;
 extern struct tseq tseq[];
-extern struct vars *v;
 extern short e820_nr;
 extern char memsz_mode;
 
@@ -147,9 +146,13 @@ void get_config()
 					cprint(POP_Y+1, POP_X+2,
 						"Test Selection:");
 					cprint(POP_Y+3, POP_X+4,
-						"Test Number [0-11]: ");
+						"Test Number [0-12]: ");
 					i = getval(POP_Y+3, POP_X+24, 0);
-					if (i <= 11) {
+					if (i <= 12) {
+						if (i != v->testsel) {
+							v->pass = 0;
+							v->ecount = 0;
+						}
 						v->testsel = i;
 					}
 					find_ticks();
