@@ -1,7 +1,7 @@
-/* init.c - MemTest-86  Version 3.0
+/* init.c - MemTest-86  Version 3.1
  *
  * Released under version 2 of the Gnu Public License.
- * By Chris Brady, cbrady@sgi.com
+ * By Chris Brady
  */
 
 #include "test.h"
@@ -11,7 +11,6 @@
 #include "pci.h"
 #include "io.h"
 
-extern struct vars *v;
 extern short memsz_mode;
 extern short firmware;
 
@@ -47,7 +46,7 @@ static void display_init(void)
 	for(i=0, pp=(char *)(SCREEN_ADR+1); i<TITLE_WIDTH; i++, pp+=2) {
 		*pp = 0x47;
 	}
-	cprint(0, 0, "      Memtest-86 v3.0       ");
+	cprint(0, 0, "      Memtest-86 v3.1       ");
 
 	/* Do reverse video for the bottom display line */
 	for(i=0, pp=(char *)(SCREEN_ADR+1+(24 * 160)); i<80; i++, pp+=2) {
@@ -375,15 +374,24 @@ static void cpu_type(void)
 			}
 			break;
 		case 6:
+			/* Get L1 & L2 cache sizes */
+			l1_cache = cpu_id.cache_info[3];
+			l1_cache += cpu_id.cache_info[7];
+			l2_cache = (cpu_id.cache_info[11] << 8);
+			l2_cache += cpu_id.cache_info[10];
+
 			switch(cpu_id.model) {
 			case 1:
 			case 2:
 			case 4:
 			case 6:
-				cprint(LINE_CPU, 0, "AMD Athlon");
-				off = 10;
-				l2_cache = (cpu_id.cache_info[11] << 8);
-				l2_cache += cpu_id.cache_info[10];
+				if (l2_cache == 64) {
+					cprint(LINE_CPU, 0, "AMD Duron");
+					off = 9;
+				} else {
+					cprint(LINE_CPU, 0, "AMD Athlon");
+					off = 10;
+				}
 				break;
 			case 3:
 			case 7:
@@ -392,16 +400,33 @@ static void cpu_type(void)
 				/* Duron stepping 0 CPUID for L2 is broken */
 				/* (AMD errata T13)*/
 				if (cpu_id.step == 0) { /* stepping 0 */
-					/* Hard code the right size */
+					/* Hard code the right L2 size */
 					l2_cache = 64;
-				} else {
-					l2_cache = (cpu_id.cache_info[11] << 8);
-					l2_cache += cpu_id.cache_info[10];
 				}
 				break;
+			case 8:
+			case 9:
+			case 10:
+				cprint(LINE_CPU, 0, "Athlon XP/MP");
+				off = 12;
 			}
+			break;
+		case 15:
+			/* Get L1 & L2 cache sizes */
 			l1_cache = cpu_id.cache_info[3];
 			l1_cache += cpu_id.cache_info[7];
+			l2_cache = (cpu_id.cache_info[11] << 8);
+			l2_cache += cpu_id.cache_info[10];
+
+			switch(cpu_id.model) {
+			case 4:
+				cprint(LINE_CPU, 0, "Athlon 64");
+				off = 9;
+				break;
+			case 5:
+				cprint(LINE_CPU, 0, "AMD Opteron");
+				off = 11;
+			}
 		}
 		break;
 
@@ -665,21 +690,40 @@ static void cpu_type(void)
 static void cacheable(void)
 {
 	ulong speed, pspeed;
-	ulong paddr, mem_top, cache_top;
+	ulong paddr, mem_top, cached;
 
 	mem_top = v->pmap[v->msegs - 1].end;
-	cache_top = mem_top;
+	cached = v->test_pages;
 	pspeed = 0;
 	for (paddr=0x200; paddr <= mem_top - 64; paddr+=0x400) {
+		int i;
+		int found;
+		/* See if the paddr is at a testable location */
+		found = 0;
+		for(i = 0; i < v->msegs; i++) {
+			if ((v->pmap[i].start >= paddr)  &&
+				(v->pmap[i].end <= (paddr + 32))) {
+				found = 1;
+				break;
+			}
+		}
+		if (!found) {
+			continue;
+		}
+		/* Map the range and perform the test */
+		map_page(paddr);
 		speed = memspeed((ulong)mapping(paddr), 32*4096, 1);
 		if (pspeed) {
 			if (speed < pspeed) {
-				cache_top = paddr;
+				cached -= 32;
 			}
 			pspeed = (ulong)((float)speed * 0.7);
 		}
 	}
-	aprint(LINE_INFO, COL_CACHE_TOP, cache_top);
+	aprint(LINE_INFO, COL_CACHE_TOP, cached);
+	/* Ensure the default set of pages are mapped */
+	map_page(0);
+	map_page(0x80000);
 }
 
 
