@@ -1,7 +1,7 @@
-/* lib.c - MemTest-86  Version 2.9
+/* lib.c - MemTest-86  Version 3.1
  *
  * Released under version 2 of the Gnu Public License.
- * By Chris Brady, cbrady@sgi.com
+ * By Chris Brady
  */
 #include "io.h"
 #include "serial.h"
@@ -9,8 +9,6 @@
 #include "config.h"
 #include "screen_buffer.h"
 
-extern struct vars *v;
-
 int slock = 0, lsr = 0;
 short serial_cons = SERIAL_CONSOLE_DEFAULT;
 char buf[18];
@@ -44,28 +42,57 @@ char *codes[] = {
 };
 
 struct eregs {
-	ulong eax;
-	ulong ebx;
-	ulong ecx;
-	ulong edx;
-	ulong edi;
-	ulong esi;
-	ulong ebp;
 	ulong esp;
+	ulong ebp;
+	ulong esi;
+	ulong edi;
+	ulong edx;
+	ulong ecx;
+	ulong ebx;
+	ulong eax;
 	ulong vect;
-	ulong eflag;
-	ulong cs;
-	ulong eip;
 	ulong code;
+	ulong eip;
+	ulong cs;
+	ulong eflag;
 };
 
-struct eregs trap_regs;
+int memcmp(const void *s1, const void *s2, ulong count)
+{
+	const unsigned char *src1 = s1, *src2 = s2;
+	int i;
+	for(i = 0; i < count; i++) {
+		if (src1[i] != src2[i]) {
+			return (int)src1[i] - (int)src2[i];
+		}
+	}
+	return 0;
+}
 
+void *memmove(void *dest, const void *src, ulong n)
+{
+	long i;
+	char *d = (char *)dest, *s = (char *)src;
+
+	/* If src == dest do nothing */
+	if (dest < src) {
+		for(i = 0; i < n; i++) {
+			d[i] = s[i];
+		}
+	}
+	else if (dest > src) {
+		for(i = n -1; i >= 0; i--) {
+			d[i] = s[i];
+		}
+	}
+	return dest;
+}
 /*
  * Scroll the error message area of the screen as needed
  * Starts at line LINE_SCROLL and ends at line 23
  */
-void scroll() {
+void scroll(void) 
+{
 	int i, j;
 	char *s, tmp;
 
@@ -99,7 +126,7 @@ void scroll() {
 /*
  * Print characters on screen
  */
-void cprint(int y, int x, char *text)
+void cprint(int y, int x, const char *text)
 {
 	register int i;
 	char *dptr;
@@ -108,9 +135,8 @@ void cprint(int y, int x, char *text)
 	for (i=0; text[i]; i++) {
 		*dptr = text[i];
 		dptr += 2;
-                set_scrn_buf(y, x+i, text[i]);
         }
-        tty_print_line(y, x, i);
+        tty_print_line(y, x, text);
 }
 
 void itoa(char s[], int n) 
@@ -132,8 +158,10 @@ void itoa(char s[], int n)
 void reverse(char s[])
 {
   int c, i, j;
+  for(j = 0; s[j] != 0; j++)
+	  ;
 
-  for(i=0, j = strlen(s) -1; i < j; i++, j--) {
+  for(i=0, j = j - 1; i < j; i++, j--) {
     c = s[i];
     s[i] = s[j];
     s[j] = c;
@@ -143,17 +171,25 @@ void reverse(char s[])
 /*
  * Print a people friendly address
  */
-void aprint(int y, int x, ulong val)
+void aprint(int y, int x, ulong page)
 {
-	if (val < 10000) {
-		dprint(y, x, val, 5, 0);
-	} else if (val < 1024*9999) {
-		dprint(y, x, val/1024, 4, 0);
+	/* page is in multiples of 4K */
+	if ((page << 2) < 9999) {
+		dprint(y, x, page << 2, 4, 0);
 		cprint(y, x+4, "K");
-	} else {
-		dprint(y, x, val/(1024*1024), 4, 0);
+	}
+	else if ((page >>8) < 9999) {
+		dprint(y, x, (page  + (1 << 7)) >> 8, 4, 0);
 		cprint(y, x+4, "M");
 	}
+	else if ((page >>18) < 9999) {
+		dprint(y, x, (page + (1 << 17)) >> 18, 4, 0);
+		cprint(y, x+4, "G");
+	}
+	else {
+		dprint(y, x, (page + (1 << 27)) >> 28, 4, 0);
+		cprint(y, x+4, "T");
+	}
 }
 
 /*
@@ -214,11 +250,11 @@ void dprint(int y, int x, ulong val, int len, int right)
 }
 
 /*
- * Print a hex number on screen
+ * Print a hex number on screen at least digits long
  */
-void hprint(int y,int x, ulong val)
+void hprint2(int y,int x, unsigned long val, int digits)
 {
-	ulong j;
+	unsigned long j;
 	int i, idx, flag = 0;
 
         for (i=0, idx=0; i<8; i++) {
@@ -236,8 +272,22 @@ void hprint(int y,int x, ulong val)
 			flag++;
 		}
         }
+	if (digits > 8) {
+		digits = 8;
+	}
+	if (flag > digits) {
+		digits = flag;
+	}
         buf[idx] = 0;
-	cprint(y,x,buf);
+	cprint(y,x,buf + (idx - digits));
+}
+
+/*
+ * Print a hex number on screen
+ */
+void hprint(int y, int x, unsigned long val)
+{
+	return hprint2(y, x, val, 8);
 }
 
 /*
@@ -258,16 +308,21 @@ void xprint(int y,int x, ulong val)
 }
 	
 /* Handle an interrupt */
-void inter()
+void inter(struct eregs *trap_regs)
 {
 	int i, line;
 	char *pp;
+	ulong address = 0;
 
+	/* Get the page fault address */
+	if (trap_regs->vect == 14) {
+		__asm__("movl %%cr2,%0":"=r" (address));
+	}
 #ifdef PARITY_MEM
 
 	/* Check for a parity error */
-	if (trap_regs.vect == 2) {
-		parity_err(trap_regs.edi, trap_regs.esi);
+	if (trap_regs->vect == 2) {
+		parity_err(trap_regs->edi, trap_regs->esi);
 		return;
 	}
 #endif
@@ -281,49 +336,69 @@ void inter()
 
 	cprint(line, 0, "Unexpected Interrupt - Halting");
 	cprint(line+2, 0, " Type: ");
-	if (trap_regs.vect <= 19) {
-		cprint(line+2, 7, codes[trap_regs.vect]);
+	if (trap_regs->vect <= 19) {
+		cprint(line+2, 7, codes[trap_regs->vect]);
 	} else {
-		hprint(line+2, 7, trap_regs.vect);
+		hprint(line+2, 7, trap_regs->vect);
 	}
 	cprint(line+3, 0, "   PC: ");
-	hprint(line+3, 7, trap_regs.eip);
+	hprint(line+3, 7, trap_regs->eip);
 	cprint(line+4, 0, "   CS: ");
-	hprint(line+4, 7, trap_regs.cs);
+	hprint(line+4, 7, trap_regs->cs);
 	cprint(line+5, 0, "Eflag: ");
-	hprint(line+5, 7, trap_regs.eflag);
+	hprint(line+5, 7, trap_regs->eflag);
 	cprint(line+6, 0, " Code: ");
-	hprint(line+6, 7, trap_regs.code);
+	hprint(line+6, 7, trap_regs->code);
+	if (trap_regs->vect == 14) {
+		/* Page fault address */
+		cprint(line+7, 0, " Addr: ");
+		hprint(line+7, 7, address);
+	}
 
 	cprint(line+2, 20, "eax: ");
-	hprint(line+2, 25, trap_regs.eax);
+	hprint(line+2, 25, trap_regs->eax);
 	cprint(line+3, 20, "ebx: ");
-	hprint(line+3, 25, trap_regs.ebx);
+	hprint(line+3, 25, trap_regs->ebx);
 	cprint(line+4, 20, "ecx: ");
-	hprint(line+4, 25, trap_regs.ecx);
+	hprint(line+4, 25, trap_regs->ecx);
 	cprint(line+5, 20, "edx: ");
-	hprint(line+5, 25, trap_regs.edx);
+	hprint(line+5, 25, trap_regs->edx);
 	cprint(line+6, 20, "edi: ");
-	hprint(line+6, 25, trap_regs.edi);
+	hprint(line+6, 25, trap_regs->edi);
 	cprint(line+7, 20, "esi: ");
-	hprint(line+7, 25, trap_regs.esi);
+	hprint(line+7, 25, trap_regs->esi);
 	cprint(line+8, 20, "ebp: ");
-	hprint(line+8, 25, trap_regs.ebp);
+	hprint(line+8, 25, trap_regs->ebp);
 	cprint(line+9, 20, "esp: ");
-	hprint(line+9, 25, trap_regs.esp);
+	hprint(line+9, 25, trap_regs->esp);
 
 	cprint(line+1, 38, "Stack:");
 	for (i=0; i<12; i++) {
-		hprint(line+2+i, 38, trap_regs.esp+(4*i));
-		hprint(line+2+i, 47, *(ulong*)(trap_regs.esp+(4*i)));
-		hprint(line+2+i, 57, trap_regs.esp+(4*(i+12)));
-		hprint(line+2+i, 66, *(ulong*)(trap_regs.esp+(4*(i+12))));
+		hprint(line+2+i, 38, trap_regs->esp+(4*i));
+		hprint(line+2+i, 47, *(ulong*)(trap_regs->esp+(4*i)));
+		hprint(line+2+i, 57, trap_regs->esp+(4*(i+12)));
+		hprint(line+2+i, 66, *(ulong*)(trap_regs->esp+(4*(i+12))));
 	}
 
-	while(1);
+	cprint(line+11, 0, "CS:EIP:                          ");
+	pp = (unsigned char *)trap_regs->eip;
+	for(i = 0; i < 10; i++) {
+		hprint2(line+11, 8+(3*i), pp[i], 2);
+	}
+
+	while(1) {
+		check_input();
+	}
 }
 
-void set_cache(int val) {
+void set_cache(int val) 
+{
+	extern struct cpu_ident cpu_id;
+	/* 386's don't have a cache */
+	if ((cpu_id.cpuid < 1) && (cpu_id.type == 3)) {
+		cprint(LINE_INFO, COL_CACHE, "none");
+		return;
+	}
 	switch(val) {
 	case 0:
 		cache_off();	
@@ -367,7 +442,7 @@ int get_key() {
 	return((c));
 }
 
-void check_input()
+void check_input(void)
 {
 	unsigned char c;
 
@@ -396,6 +471,10 @@ void check_input()
 			slock = 1;
 			footer();
 			break;
+		case 0x26:
+			/* ^L/L - redraw the display */
+			tty_print_screen();
+			break;
 		}
 	}
 }
@@ -410,111 +489,167 @@ void footer()
 	}
 }
 
-ulong getval(x, y)
+ulong getval(int x, int y, int result_shift)
 {
-	int flag=0, c, n=0;
-	ulong i, val, mult;
+	unsigned long val;
+	int done;
+	int c;
+	int i, n;
+	int base;
+	int shift;
 	char buf[16];
-	char str[2];
 
-	str[1] = 0;
+	for(i = 0; i < sizeof(buf)/sizeof(buf[0]); i++ ) {
+		buf[i] = ' ';
+	}
+	buf[sizeof(buf)/sizeof(buf[0]) -1] = '\0';
+	
 	wait_keyup();
-	while (!flag) {
+	done = 0;
+	n = 0;
+	base = 10;
+	while(!done) {
+		/* Read a new character and process it */
 		c = get_key();
-
-		/* If CR and something has been entered, bail */
-		if (c == 28 && n) {
+		switch(c) {
+		case 0x26: /* ^L/L - redraw the display */
+			tty_print_screen();
+			break;
+		case 0x1c: /* CR */
+			/* If something has been entered we are done */
+			if(n) done = 1;
+			break;
+		case 0x19: /* p */ buf[n] = 'p'; break;
+		case 0x22: /* g */ buf[n] = 'g'; break;
+		case 0x32: /* m */ buf[n] = 'm'; break;
+		case 0x25: /* k */ buf[n] = 'k'; break;
+		case 0x2d: /* x */
+			/* Only allow 'x' after an initial 0 */
+			if (n == 1 && (buf[0] == '0')) {
+				buf[n] = 'x';
+			}
+			break;
+		case 0x0e: /* BS */
+			if (n > 0) {
+				n -= 1;
+				buf[n] = ' ';
+			}
+			break;
+		/* Don't allow entering a number not in our current base */
+		case 0x0B: if (base >= 1) buf[n] = '0'; break;
+		case 0x02: if (base >= 2) buf[n] = '1'; break;
+		case 0x03: if (base >= 3) buf[n] = '2'; break;
+		case 0x04: if (base >= 4) buf[n] = '3'; break;
+		case 0x05: if (base >= 5) buf[n] = '4'; break;
+		case 0x06: if (base >= 6) buf[n] = '5'; break;
+		case 0x07: if (base >= 7) buf[n] = '6'; break;
+		case 0x08: if (base >= 8) buf[n] = '7'; break;
+		case 0x09: if (base >= 9) buf[n] = '8'; break;
+		case 0x0A: if (base >= 10) buf[n] = '9'; break;
+		case 0x1e: if (base >= 11) buf[n] = 'a'; break;
+		case 0x30: if (base >= 12) buf[n] = 'b'; break;
+		case 0x2e: if (base >= 13) buf[n] = 'c'; break;
+		case 0x20: if (base >= 14) buf[n] = 'd'; break;
+		case 0x12: if (base >= 15) buf[n] = 'e'; break;
+		case 0x21: if (base >= 16) buf[n] = 'f'; break;
+		default:
 			break;
 		}
-
-		/* M */
-		if (c == 50 && n) {
-			cprint(x, y+n, "m");
-			buf[n] = 'm';
-			if (n < 12) {
-				n++;
-			}
-			continue;
+		/* Don't allow anything to be entered after a suffix */
+		if (n > 0 && (
+			(buf[n-1] == 'p') || (buf[n-1] == 'g') || 
+			(buf[n-1] == 'm') || (buf[n-1] == 'k'))) {
+			buf[n] = ' ';
 		}
-
-		/* K */
-		if (c == 37 && n) {
-			cprint(x, y+n, "k");
-			buf[n] = 'k';
-			if (n < 12) {
-				n++;
-			}
-			continue;
-		}
-
-		/* Backspace */
-		if (c == 14 && n) {
-			n--;
-			str[0] = ' ';
-			cprint(x, y+n, str);
-		}
-
-		c--;
-		if (c < 1 || c > 10) {
-			continue;
-		}
-		if (c == 10) {
-			c = 0;
-		}
-
-		str [0] = '0'+c;
-		cprint(x, y+n, str);
-		buf[n] = c;
-		if (n < 12) {
+		/* If we have entered a character increment n */
+		if (buf[n] != ' ') {
 			n++;
 		}
-	}
+		buf[n] = ' ';
+		/* Print the current number */
+		cprint(x, y, buf);
 
-	/* Convert stored values to an integer */
-	i = 1;
+		/* Find the base we are entering numbers in */
+		base = 10;
+		if ((buf[0] == '0') && (buf[1] == 'x')) {
+			base = 16;
+		}
+		else if (buf[0] == '0') {
+			base = 8;
+		}
+	}
+	/* Compute our current shift */
+	shift = 0;
+	switch(buf[n-1]) {
+	case 'g': /* gig */  shift = 30; break;
+	case 'm': /* meg */  shift = 20; break;
+	case 'p': /* page */ shift = 12; break;
+	case 'k': /* kilo */ shift = 10; break;
+	}
+	shift -= result_shift;
+
+	/* Compute our current value */
 	val = 0;
-	mult = 1;
-	while (n--) {
-		if (buf[n] == 'm') {
-			mult = 1048576;
-			continue;
+	for(i = (base == 16)? 2: 0; i < n; i++) {
+		unsigned long digit = 0;
+		if ((buf[i] >= '0') && (buf[i] <= '9')) {
+			digit = buf[i] - '0';
 		}
-		if (buf[n] == 'k') {
-			mult = 1024;
-			continue;
+		else if ((buf[i] >= 'a') && (buf[i] <= 'f')) {
+			digit = buf[i] - 'a' + 10;
 		}
-
-		val += buf[n] * i;
-		i *= 10;
+		else {
+				/* It must be a suffix byte */
+			break;
+		}
+		val = (val * base) + digit;
 	}
-	return(val * mult);
+	if (shift > 0) {
+		if (shift >= 32) {
+			val = 0xffffffff;
+		} else {
+			val <<= shift;
+		}
+	} else {
+		if (-shift >= 32) {
+			val = 0;
+		}
+		else {
+			val >>= -shift;
+		}
+	}
+	return val;
 }
 
-void ttyprint(int y, int x, char *p)
+void ttyprint(int y, int x, const char *p)
 {
-       static char sx[3];
-       static char sy[3];
-
-       sx[0]='\0';
-       sy[0]='\0';
-       x++; y++;
-       itoa(sx, x);
-       itoa(sy, y);
-       serial_echo_print("[");
-       serial_echo_print(sy);
-       serial_echo_print(";");
-       serial_echo_print(sx);
-       serial_echo_print("H");
-       serial_echo_print(p);
+	static char sx[3];
+	static char sy[3];
+	
+	sx[0]='\0';
+	sy[0]='\0';
+	x++; y++;
+	itoa(sx, x);
+	itoa(sy, y);
+	serial_echo_print("[");
+	serial_echo_print(sy);
+	serial_echo_print(";");
+	serial_echo_print(sx);
+	serial_echo_print("H");
+	serial_echo_print(p);
 }
 
+#if defined(SERIAL_BAUD_RATE)
+
 #if ((115200%SERIAL_BAUD_RATE) != 0)
 #error Bad ttys0 baud rate
 #endif
 
 #define SERIAL_DIV     (115200/SERIAL_BAUD_RATE)
 
-void serial_echo_init()
+#endif /* SERIAL_BAUD_RATE */
+
+void serial_echo_init(void)
 {
 	int comstat, hi, lo;
 	
@@ -527,10 +662,12 @@ void serial_echo_init()
 
 	/* now do hardwired init */
 	serial_echo_outb(0x03, UART_LCR); /* No parity, 8 data bits, 1 stop */
+#if defined(SERIAL_BAUD_RATE)
 	serial_echo_outb(0x83, UART_LCR); /* Access divisor latch */
 	serial_echo_outb(SERIAL_DIV & 0xff, UART_DLL);  /* baud rate divisor */
 	serial_echo_outb((SERIAL_DIV>> 8) & 0xff, UART_DLM);
 	serial_echo_outb(0x03, UART_LCR); /* Done with divisor */
+#endif
 
 	/* Prior to disabling interrupts, read the LSR and RBR
 	 * registers */
@@ -543,7 +680,7 @@ void serial_echo_init()
 	return;
 }
 
-void serial_echo_print(char *p)
+void serial_echo_print(const char *p)
 {
 	if (!serial_cons) {
 		return;
@@ -562,41 +699,155 @@ void serial_echo_print(char *p)
 	}
 }
 
-/* This map needs to be updated whenever a new keycode
-   is searched for.  I don't have a complete keycode
-   map, so I don't know how to fill it all in!
-*/
-
+/* Except for multi-character key sequences this mapping
+ * table is complete.  So it should not need to be updated
+ * when new keys are searched for.  However the key handling
+ * should really be turned around and only in get_key should
+ * we worry about the exact keycode that was pressed.  Everywhere
+ * else we should switch on the character...
+ */
 struct ascii_map_str ser_map[] =
 /*ascii keycode     ascii  keycode*/
-{ {  99,     46}, /* c    -> c    */
-  {  67,     46}, /* C    -> c    */
-  {  27,      1}, /* ESC  -> ESC  */
-  {  10,     28}, /* NL   -> CR   */
-  {  13,     28}, /* CR   -> CR   */
-  {  32,     28}, /* SPACE-> CR   */
-  { 109,     50}, /* m    -> m    */
-  {  77,     50}, /* M    -> M    */
-  { 110,     49}, /* n    -> n    */
-  {  78,     49}, /* N    -> n    */
-  { 107,     37}, /* k    -> k    */
-  {  75,     37}, /* K    -> K    */
-  {  87,     21}, /* y    -> y    */
-  { 119,     21}, /* Y    -> y    */
-  {   8,     14}, /* BS   -> BS   */
-  { 127,     14}, /* DEL  -> BS   */
-  {  48,     11}, /* 0    -> 0    */
-  {  49,      2}, /* 1    -> 1    */
-  {  50,      3}, /* 2    -> 2    */
-  {  51,      4}, /* 3    -> 3    */
-  {  52,      5}, /* 4    -> 4    */
-  {  53,      6}, /* 5    -> 5    */
-  {  54,      7}, /* 6    -> 6    */
-  {  55,      8}, /* 7    -> 7    */
-  {  56,      9}, /* 8    -> 8    */
-  {  57,     10}, /* 9    -> 9    */
-  {  19,     57}, /* ^S   -> SP   */
-  {  17,     28}, /* ^Q   -> CR   */
+{ 
+  /* Special cases come first so I can leave
+   * their ``normal'' mapping in the table,
+   * without it being activated.
+   */
+  {  27,   0x01}, /* ^[/ESC -> ESC  */
+  { 127,   0x0e}, /*    DEL -> BS   */
+  {   8,   0x0e}, /* ^H/BS  -> BS   */
+  {  10,   0x1c}, /* ^L/NL  -> CR   */
+  {  13,   0x1c}, /* ^M/CR  -> CR   */
+  {   9,   0x0f}, /* ^I/TAB -> TAB  */
+  {  19,   0x39}, /* ^S     -> SP   */
+  {  17,     28}, /* ^Q     -> CR   */
+
+  { ' ',   0x39}, /*     SP -> SP   */
+  { 'a',   0x1e},
+  { 'A',   0x1e},
+  {   1,   0x1e}, /* ^A      -> A */
+  { 'b',   0x30},
+  { 'B',   0x30},
+  {   2,   0x30}, /* ^B      -> B */
+  { 'c',   0x2e},
+  { 'C',   0x2e},
+  {   3,   0x2e}, /* ^C      -> C */
+  { 'd',   0x20},
+  { 'D',   0x20},
+  {   4,   0x20}, /* ^D      -> D */
+  { 'e',   0x12},
+  { 'E',   0x12},
+  {   5,   0x12}, /* ^E      -> E */
+  { 'f',   0x21},
+  { 'F',   0x21},
+  {   6,   0x21}, /* ^F      -> F */
+  { 'g',   0x22},
+  { 'G',   0x22},
+  {   7,   0x22}, /* ^G      -> G */
+  { 'h',   0x23},
+  { 'H',   0x23},
+  {   8,   0x23}, /* ^H      -> H */
+  { 'i',   0x17},
+  { 'I',   0x17},
+  {   9,   0x17}, /* ^I      -> I */
+  { 'j',   0x24},
+  { 'J',   0x24},
+  {  10,   0x24}, /* ^J      -> J */
+  { 'k',   0x25},
+  { 'K',   0x25},
+  {  11,   0x25}, /* ^K      -> K */
+  { 'l',   0x26},
+  { 'L',   0x26},
+  {  12,   0x26}, /* ^L      -> L */
+  { 'm',   0x32},
+  { 'M',   0x32},
+  {  13,   0x32}, /* ^M      -> M */
+  { 'n',   0x31},
+  { 'N',   0x31},
+  {  14,   0x31}, /* ^N      -> N */
+  { 'o',   0x18},
+  { 'O',   0x18},
+  {  15,   0x18}, /* ^O      -> O */
+  { 'p',   0x19},
+  { 'P',   0x19},
+  {  16,   0x19}, /* ^P      -> P */
+  { 'q',   0x10},
+  { 'Q',   0x10},
+  {  17,   0x10}, /* ^Q      -> Q */
+  { 'r',   0x13},
+  { 'R',   0x13},
+  {  18,   0x13}, /* ^R      -> R */
+  { 's',   0x1f},
+  { 'S',   0x1f},
+  {  19,   0x1f}, /* ^S      -> S */
+  { 't',   0x14},
+  { 'T',   0x14},
+  {  20,   0x14}, /* ^T      -> T */
+  { 'u',   0x16},
+  { 'U',   0x16},
+  {  21,   0x16}, /* ^U      -> U */
+  { 'v',   0x2f},
+  { 'V',   0x2f},
+  {  22,   0x2f}, /* ^V      -> V */
+  { 'w',   0x11},
+  { 'W',   0x11},
+  {  23,   0x11}, /* ^W      -> W */
+  { 'x',   0x2d},
+  { 'X',   0x2d},
+  {  24,   0x2d}, /* ^X      -> X */
+  { 'y',   0x15},
+  { 'Y',   0x15},
+  {  25,   0x15}, /* ^Y      -> Y */
+  { 'z',   0x2c},
+  { 'Z',   0x2c},
+  {  26,   0x2c}, /* ^Z      -> Z */
+  { '-',   0x0c},
+  { '_',   0x0c},
+  {  31,   0x0c}, /* ^_      -> _ */
+  { '=',   0x0c},
+  { '+',   0x0c},
+  { '[',   0x1a},
+  { '{',   0x1a},
+  {  27,   0x1a}, /* ^[      -> [ */
+  { ']',   0x1b},
+  { '}',   0x1b},
+  {  29,   0x1b}, /* ^]      -> ] */
+  { ';',   0x27},
+  { ':',   0x27},
+  { '\'',  0x28},
+  { '"',   0x28},
+  { '`',   0x29},
+  { '~',   0x29},
+  { '\\',  0x2b},
+  { '|',   0x2b},
+  {  28,   0x2b}, /* ^\      -> \ */
+  { ',',   0x33},
+  { '<',   0x33},
+  { '.',   0x34},
+  { '>',   0x34},
+  { '/',   0x35},
+  { '?',   0x35},
+  { '1',   0x02},
+  { '!',   0x02},
+  { '2',   0x03},
+  { '@',   0x03},
+  { '3',   0x04},
+  { '#',   0x04},
+  { '4',   0x05},
+  { '$',   0x05},
+  { '5',   0x06},
+  { '%',   0x06},
+  { '6',   0x07},
+  { '^',   0x07},
+  {  30,   0x07}, /* ^^      -> 6 */
+  { '7',   0x08},
+  { '&',   0x08},
+  { '8',   0x09},
+  { '*',   0x09},
+  { '9',   0x0a},
+  { '(',   0x0a},
+  { '0',   0x0b},
+  { ')',   0x0b},
   {   0,      0}
 };
 
