@@ -1,7 +1,7 @@
-/* main.c - MemTest-86  Version 3.0
+/* main.c - MemTest-86  Version 3.1
  *
  * Released under version 2 of the Gnu Public License.
- * By Chris Brady, cbrady@sgi.com
+ * By Chris Brady
  */
 #include "test.h"
 #include "defs.h"
@@ -23,6 +23,7 @@ const struct tseq tseq[] = {
 	{0, 1, 2, 80, 0,   "[Moving inv, 8 bit pattern, no cache] "},
 	{1, 4, 2, 1280, 0, "[Modulo 20, 8 bit pattern , cached]   "},
 	{0, 2, 2, 320, 0,  "[Moving inv, 32 bit pattern, no cache]"},
+	{1, 8, 1, 1, 0,    "[Bit fade test, 90 min, 2 patterns]   "},
 	{0, 0, 0, 0, 0, NULL}
 };
 
@@ -330,6 +331,9 @@ void do_test(void)
 	case 7: /* Block move test */
 		block_move(tseq[v->test].iter);
 		break;
+	case 8: /* Bit fade test */
+		bit_fade();
+		break;
 	}
  skip_window:
 	if (bail) {
