@@ -1,7 +1,7 @@
-/* test.c - MemTest-86  Version 3.0
+/* test.c - MemTest-86  Version 3.1
  *
  * Released under version 2 of the Gnu Public License.
- * By Chris Brady, cbrady@sgi.com
+ * By Chris Brady
  */
 #include "test.h"
 #include "config.h"
@@ -9,7 +9,6 @@
 extern int segs, bail;
 extern volatile ulong *p;
 extern ulong p1, p2;
-extern struct vars *v;
 extern int test_ticks, nticks;
 extern struct tseq tseq[];
 void poll_errors();
@@ -1116,6 +1115,60 @@ void block_move(int iter)
 	}
 }
 
+/*
+ * Test memory for bit fade.
+ */
+void bit_fade()
+{
+	int j;
+	volatile ulong *pe;
+	volatile ulong bad;
+	volatile ulong *start,*end;
+
+	/* Do -1 and 0 patterns */
+	p1 = 0;
+	while (1) {
+
+		/* Display the current pattern */
+		hprint(LINE_PAT, COL_PAT, p1);
+
+		/* Initialize memory with the initial pattern.  */
+		for (j=0; j<segs; j++) {
+			start = v->map[j].start;
+			end = v->map[j].end;
+			pe = start;
+			p = start;
+			for (p=end; p<end; p++) {
+				*p = p1;
+			}
+			do_tick();
+			BAILR
+		}
+		/* Snooze for 90 minutes */
+		sleep (5400);
+
+		/* Make sure that nothing changed while sleeping */
+		for (j=0; j<segs; j++) {
+			start = v->map[j].start;
+			end = v->map[j].end;
+			pe = start;
+			p = start;
+			for (p=end; p<end; p++) {
+ 				if ((bad=*p) != p1) {
+					error((ulong*)p, p1, bad);
+				}
+			}
+			do_tick();
+			BAILR
+		}
+		if (p1 == 0) {
+			p1=-1;
+		} else {
+			break;
+		}
+	}
+}
+
 /*
  * Display data error message. Don't display duplicate errors.
  */
@@ -1282,7 +1335,7 @@ void print_err( ulong *adr, ulong good, ulong bad, ulong xor)
  * Print an ecc error
  */
 void print_ecc_err(unsigned long page, unsigned long offset, 
-	int corrected, unsigned char syndrom, int channel)
+	int corrected, unsigned short syndrome, int channel)
 {
 	update_err_counts();
 	++(v->ecc_ecount);
@@ -1293,7 +1346,7 @@ void print_ecc_err(unsigned long page, unsigned long offset,
 
 	cprint(v->msg_line, 36, 
 		corrected?"corrected           ": "uncorrected         ");
-	hprint2(v->msg_line, 56, syndrom, 2);
+	hprint2(v->msg_line, 56, syndrome, 4);
 	cprint(v->msg_line, 66, "ECC"); 
 	dprint(v->msg_line, 71, channel, 2, 0);
 }
@@ -1423,3 +1476,44 @@ void do_tick(void)
 	/* Poll for ECC errors */
 	poll_errors();
 }
+
+void sleep(int n)
+{
+	int i;
+	ulong sh, sl, l, h, t;
+
+	/* save the starting time */
+	asm __volatile__(
+		"rdtsc":"=a" (sl),"=d" (sh));
+
+	/* loop for n seconds */
+	while (1) {
+		asm __volatile__(
+			"rdtsc":"=a" (l),"=d" (h));
+		asm __volatile__ (
+			"subl %2,%0\n\t"
+			"sbbl %3,%1"
+			:"=a" (l), "=d" (h)
+			:"g" (sl), "g" (sh),
+			"0" (l), "1" (h));
+		t = h * ((unsigned)0xffffffff / v->clks_msec) / 1000;
+		t += (l / v->clks_msec) / 1000;
+
+		/* Is the time up? */
+		if (t >= n) {
+			break;
+		}
+
+		/* Display the elapsed time on the screen */
+		i = t % 60;
+		dprint(LINE_TIME, COL_TIME+9, i%10, 1, 0);
+		dprint(LINE_TIME, COL_TIME+8, i/10, 1, 0);
+		t /= 60;
+		i = t % 60;
+		dprint(LINE_TIME, COL_TIME+6, i % 10, 1, 0);
+		dprint(LINE_TIME, COL_TIME+5, i / 10, 1, 0);
+		t /= 60;
+		dprint(LINE_TIME, COL_TIME, t, 4, 0);
+		BAILR
+	}
+}
