@@ -1,7 +1,7 @@
-/* test.h - MemTest-86  Version 3.0
+/* test.h - MemTest-86  Version 3.1
  *
  * Released under version 2 of the Gnu Public License.
- * By Chris Brady, cbrady@sgi.com
+ * By Chris Brady
  */
 
 
@@ -44,6 +44,7 @@ typedef unsigned long ulong;
 #define RES_START	0xa0000
 #define RES_END		0x100000
 #define SCREEN_ADR	0xb8000
+#define SCREEN_END_ADR  (SCREEN_ADR + 80*25*2)
 
 #define TITLE_WIDTH	28
 #define LINE_TIME	9
@@ -125,11 +126,13 @@ void get_config(void);
 void get_printmode(void);
 void addr_tst1(void);
 void addr_tst2(void);
+void bit_fade(void);
+void sleep(int sec);
 void block_move(int iter);
 void find_ticks(void);
 void print_err(ulong *adr, ulong good, ulong bad, ulong xor);
 void print_ecc_err(ulong page, ulong offset, int corrected, 
-	unsigned char syndrom, int channel);
+	unsigned short syndrome, int channel);
 void mem_size(void);
 void adj_mem(void);
 ulong getval(int x, int y, int result_shift);
